package svc

import (
	"applet/app/utils"
	"applet/app/utils/logx"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_models.git/utils/logx"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/enum"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/svc"
	"fmt"
	"time"
	"xorm.io/xorm"
)

func DealPlatformRevenueData(eg *xorm.Engine, dbName string) {
	fmt.Println("deal_platform_revenue_data...")
	defer func() {
		if err := recover(); err != nil {
			_ = logx.Error(err)
		}
	}()

	platformRevenueDataDb := implement.NewPlatformRevenueDataDb(eg)
	platformRevenueDataList, err := platformRevenueDataDb.PlatformRevenueDataFindNotFinish()
	if err != nil {
		_ = logx.Error(err)
		return
	}

	now := time.Now()
	recordsDb := implement.NewPlatformRevenueDataRecordsDb(eg)
	for _, data := range platformRevenueDataList {
		// 判断是否是第一次执行
		lastRecord, err1 := recordsDb.PlatformRevenueDataRecordsGetLast(data.Id)
		if err1 != nil {
			_ = logx.Error(err1)
			continue
		}
		if lastRecord == nil {
			// 首次执行 判断当前是否可以执行
			if utils.TimeParseStd(data.CreateAt).Add(time.Duration(data.Frequency) * time.Minute).After(now) {
				continue
			}
		} else {
			// 已经被执行过
			if utils.TimeParseStd(lastRecord.CreateAt).Add(time.Duration(data.Frequency) * time.Minute).After(now) {
				continue
			}
		}

		// 总执行次数 按十分钟更新
		times := data.Hours * 60 / data.Frequency
		// 每次更新金额
		amount := utils.AnyToFloat64(data.TotalAmount) / utils.AnyToFloat64(times)

		session := eg.NewSession()
		defer func() {
			session.Close()
			if err := recover(); err != nil {
				_ = zhios_order_relate_logx.Error(err)
			}
		}()
		session.Begin()

		eggEnergyCoreData, cb, err2 := svc.GetEggEnergyCoreData(eg)
		if err2 != nil {
			_ = logx.Error(err2)
		}
		if cb != nil {
			defer cb() // 释放锁
		}
		//计算涨价公式
		err3, calcPriceIncreaseFormulaResp := egg_energy.CalcPriceIncreaseFormula(utils.AnyToString(amount), eggEnergyCoreData)
		if err3 != nil {
			_ = logx.Error(err3)
			_ = session.Rollback()
			continue
		}

		dealAvailableEggEnergyCoinReq := md2.DealAvailableEggEnergyCoinReq{
			Amount:                 calcPriceIncreaseFormulaResp.GetEggEnergyNums,
			AmountFee:              "",
			BeforePrice:            calcPriceIncreaseFormulaResp.BeforePrice,
			AfterPrice:             calcPriceIncreaseFormulaResp.AfterPrice,
			BeforePlanetTotalValue: calcPriceIncreaseFormulaResp.BeforePlanetTotalValue,
			AfterPlanetTotalValue:  calcPriceIncreaseFormulaResp.AfterPlanetTotalValue,
			BeforeEnergyTotalNums:  calcPriceIncreaseFormulaResp.BeforeEnergyTotalNums,
			AfterEnergyTotalNums:   calcPriceIncreaseFormulaResp.AfterEnergyTotalNums,
		}

		err4 := egg_energy.DealAvailableEggEnergyCoin(session, int(enum.CapitalInjection), eggEnergyCoreData, dealAvailableEggEnergyCoinReq)
		if err4 != nil {
			fmt.Println("ActivityCoinAutoExchangeEggPersonEnergy:::::err111:::", err4)
			_ = session.Rollback()
			continue
		}

		// 插入平台营收详细数据
		record := model.PlatformRevenueDataRecords{
			RecordsId:              data.Id,
			TotalAmount:            data.TotalAmount,
			BalanceAmount:          utils.AnyToString(utils.AnyToFloat64(data.BalanceAmount) - amount),
			BalanceTimes:           data.BalanceTimes - 1,
			BeforePrice:            calcPriceIncreaseFormulaResp.BeforePrice,
			AfterPrice:             calcPriceIncreaseFormulaResp.AfterPrice,
			BeforePlanetTotalValue: calcPriceIncreaseFormulaResp.BeforePlanetTotalValue,
			AfterPlanetTotalValue:  calcPriceIncreaseFormulaResp.AfterPlanetTotalValue,
			CreateAt:               now.Format("2006-01-02 15:04:05"),
			UpdateAt:               now.Format("2006-01-02 15:04:05"),
		}
		_, err5 := recordsDb.PlatformRevenueDataRecordsInsertBySession(session, record)
		if err5 != nil {
			_ = session.Rollback()
			continue
		}

		// 更新当前数据
		platformRevenueData := model.PlatformRevenueData{
			Id:            data.Id,
			Kind:          data.Kind,
			TotalAmount:   data.TotalAmount,
			BalanceAmount: utils.AnyToString(utils.AnyToFloat64(data.BalanceAmount) - amount),
			Hours:         data.Hours,
			BalanceTimes:  data.BalanceTimes - 1,
			Frequency:     data.Frequency,
			Memo:          data.Memo,
			CreateAt:      data.CreateAt,
			UpdateAt:      now.Format("2006-01-02 15:04:05"),
		}

		forceColumns := []string{
			"balance_times",
			"balance_amount",
		}
		_, err6 := platformRevenueDataDb.PlatformRevenueDataUpdateBySession(session, platformRevenueData, forceColumns...)
		if err6 != nil {
			_ = logx.Error(err6)
			_ = session.Rollback()
			continue
		}
	}
}
